'use strict';

const argv = require('optimist').argv;
const fs = require('fs');

const fileArg = argv.file || argv.f;
const statsArg = argv.stats || argv.s;
const maxCleanWordsArg = argv.max || argv.m || 4;
const commaChanceArg = argv.comma || argv.c || 5;

const wordsExpression = /([\w]+\s[\u2014\-]\s)|([\w]+[\s,.?!&]+)|(\w+\-\w+[\s,.?!&]+)|(\w+['`\w]+[\s,.?!&]+)/g;

Promise.resolve()
  .then(read)
  .then(sortWords)
  .then(generateRandomSentence)
  .then(speak)
  .catch(e => console.error(e));

function read() {
  let text = fs.readFileSync(getFilePath(), 'utf-8');
  return text.match(wordsExpression).map(fixSpaces);
}

function speak(text) {
  console.log(text);
}

function sortWords(words) {
  let i, word;
  let capitalized = [];
  let withComma = [];
  let withEnding = [];
  let clean = [];

  for (i = 0; i < words.length; i++) {
    word = words[i];
    if (word.charAt(0).match(/[A-Z]/)) {
      capitalized.push(word.replace(/[^\w\s]/g, ''));
    } else if (word.charAt(word.length - 2).match(/[,]/)) {
      withComma.push(word);
    } else if (word.charAt(word.length - 2).match(/[.?!]/)) {
      withEnding.push(word);
    } else {
      clean.push(word);
    }
  }

  if (statsArg) {
    console.log('Statistics:');
    console.log(capitalized.length + ' capitalized words');
    console.log(withComma.length + ' words with comma');
    console.log(withEnding.length + ' words with period, exclamation or question marks');
    console.log(clean.length + ' clean words');
    console.log('');
    console.log(words.length + ' words total');
    console.log('-------');
  }

  return {
    capitalized: capitalized,
    withComma: withComma,
    clean: clean,
    withEnding: withEnding
  };
}

function generateRandomSentence(sortedWords) {
  let i;
  let sentence = '';
  let shouldContinue = true;
  sentence += getRandomWordFrom(sortedWords.capitalized);
  while (shouldContinue) {
    for (i = Math.ceil(Math.random() * maxCleanWordsArg); i > 0; i--) {
      sentence += getRandomWordFrom(sortedWords.clean);
    }

    if (shouldContinue = Math.random() < getCommaChance()) {
      sentence += getRandomWordFrom(sortedWords.withComma);
    }
  }
  sentence += getRandomWordFrom(sortedWords.withEnding);
  return sentence;
}

function getRandomWordFrom(words) {
  return words[Math.floor(Math.random() * words.length)];
}

function getFilePath() {
  if (fileArg) {
    return fileArg[0] === '/' || '~' ? fileArg : process.env.PWD + '/' + fileArg;
  } else {
    return __dirname + '/../books/default.txt';
  }
}

function fixSpaces(word) {
  word = word.replace(/([\s]+)/g, ' ').replace(/([\n]+)/g, '');
  word = word.charAt(word.length - 1) === ' ' ? word : word + ' ';
  return word;
}

function getCommaChance() {
  return (commaChanceArg > 9 ? 9 : commaChanceArg) / 10;
}